from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request


class OriginalHostMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        original_host = request.headers.get("X-Original-Host")

        if original_host:
            new_url = request.url.replace(netloc=original_host)
            request._url = new_url

            for i, el in enumerate(request.scope["headers"]):
                k, v = el
                key = k.decode("utf-8")

                if key.lower() in ["host", "x-forwarded-host"]:
                    request.scope["headers"][i] = (k, original_host.encode("utf-8"))

        # Proceed with the next middleware or request handler
        response = await call_next(request)
        return response



import os

from fastapi_mongo_base.core.config import Settings as BaseSettings


class Settings(BaseSettings):
    business_domains_url: str = (
        os.getenv(
            "UFAAS_BUSINESS_DOMAINS_URL",
            "https://business.ufaas.io/api/v1/apps/business",
        )
        + "/businesses/"
    )
    usso_app_auth_url: str = os.getenv(
        "USSO_APP_AUTH_URL", "https://sso.ufaas.io/app-auth/access"
    )

    app_id: str = os.getenv("APP_ID")
    app_secret: str = os.getenv("APP_SECRET")
    app_scopes: str = os.getenv("APP_SCOPES", default="[]")
    app_auth_expiry: int = 60  # 1 minute



import hashlib
import hmac
from datetime import datetime

from aiocache import cached
from fastapi_mongo_base.utils.aionetwork import aio_request
from pydantic import BaseModel, field_validator

try:
    from server.config import Settings
except ImportError:
    from .config import Settings


class AppAuth(BaseModel):
    # app_secret: str
    app_id: str
    scopes: list[str]
    timestamp: float
    sso_url: str
    secret: str | None = None

    @field_validator("timestamp")
    def check_timestamp(cls, v: int):
        if datetime.now().timestamp() - v > 60:
            raise ValueError("Timestamp expired.")
        return v

    @property
    def hash_key_part(self):
        scopes_hash = hashlib.sha256("".join(self.scopes).encode()).hexdigest()
        return f"{self.app_id}{scopes_hash}{self.timestamp}{self.sso_url}"

    def check_secret(self, app_secret: bytes | str):
        if type(app_secret) != str:
            app_secret = app_secret.decode("utf-8")

        key = f"{self.hash_key_part}{app_secret}"
        return hmac.compare_digest(
            self.secret, hashlib.sha256(key.encode()).hexdigest()
        )

    def get_secret(self, app_secret: bytes | str):
        if type(app_secret) != str:
            app_secret = app_secret.decode("utf-8")

        key = f"{self.hash_key_part}{app_secret}"
        return hashlib.sha256(key.encode()).hexdigest()


@cached(ttl=10 * 60)
async def get_access_token(
    app_id: str, app_secret, business_sso_url: str, scopes: list[str]
):
    app_auth = AppAuth(
        app_id=app_id,
        scopes=scopes,
        timestamp=datetime.now().timestamp(),
        sso_url=business_sso_url,
    )
    app_auth.secret = app_auth.get_secret(app_secret)
    response = await aio_request(
        method="post",
        url=Settings.usso_app_auth_url,
        json=app_auth.model_dump(),
    )

    return response



from enum import Enum


class Currency(str, Enum):
    none = "none"

    IRR = "IRR"
    IRT = "IRT"

    USD = "USD"
    EUR = "EUR"
    GBP = "GBP"

    USDT = "USDT"
    BTC = "BTC"
    ETH = "ETH"



import hashlib
import hmac
import json
import uuid
from datetime import datetime
from urllib.parse import urlparse

from fastapi_mongo_base.schemas import OwnedEntitySchema
from pydantic import BaseModel, Field, field_validator, model_validator
from usso.core import JWTConfig

try:
    from server.config import Settings
except ImportError:
    from .core.config import Settings


class Config(BaseModel):
    core_url: str = getattr(Settings, "core_url", "https://core.ufaas.io/")
    api_os_url: str = getattr(
        Settings, "api_os_url", "https://core.ufaas.io/api/v1/apps"
    )
    sso_url: str = getattr(Settings, "sso_url", "https://sso.ufaas.io/")
    core_sso_url: str = getattr(
        Settings, "core_sso_url", "https://sso.ufaas.io/app-auth/access"
    )

    allowed_origins: list[str] = []
    jwt_config: JWTConfig = JWTConfig(**json.loads(Settings.JWT_CONFIG))
    default_currency: str = "IRR"
    wallet_id: uuid.UUID | None = None

    def __hash__(self):
        return hash(self.model_dump_json())


class BusinessSchema(OwnedEntitySchema):
    name: str
    domain: str
    main_domain: str | None = None

    description: str | None = None
    config: Config = Config()

    @model_validator(mode="before")
    def validate_domain(cls, data: dict):
        if not data.get("domain"):
            config = data.get("config", Config())
            if isinstance(config, dict):
                config = Config(**config)
            netloc = urlparse(config.core_url).netloc
            business_name_domain = f"{data.get('name')}.{netloc}"
            data["domain"] = business_name_domain

        return data

    @property
    def refresh_url(self):
        refresh_url = self.cls_refresh_url()
        if refresh_url:
            return refresh_url
        return f"{self.config.sso_url}/auth/refresh"

    @classmethod
    def cls_refresh_url(cls):
        if hasattr(Settings, "sso_refresh_url"):
            return Settings.sso_refresh_url
        if hasattr(Settings, "USSO_URL"):
            return f"{Settings.USSO_URL}/auth/refresh"


class AppAuth(BaseModel):
    # app_secret: str
    app_id: str
    scopes: list[str]
    timestamp: int = Field(default_factory=lambda: int(datetime.now().timestamp()))
    sso_url: str
    secret: str | None = None

    @field_validator("timestamp")
    def check_timestamp(cls, v: int):
        if datetime.now().timestamp() - v > getattr(Settings, "app_auth_expiry", 60):
            raise ValueError("Timestamp expired.")
        return v

    @property
    def hash_key_part(self):
        scopes_hash = hashlib.sha256("".join(self.scopes).encode()).hexdigest()
        return f"{self.app_id}{scopes_hash}{self.timestamp}{self.sso_url}"

    def check_secret(self, app_secret: bytes | str):
        if type(app_secret) != str:
            app_secret = app_secret.decode("utf-8")

        key = f"{self.hash_key_part}{app_secret}"
        return hmac.compare_digest(
            self.secret, hashlib.sha256(key.encode()).hexdigest()
        )

    def get_secret(self, app_secret: bytes | str):
        if type(app_secret) != str:
            app_secret = app_secret.decode("utf-8")

        key = f"{self.hash_key_part}{app_secret}"
        return hashlib.sha256(key.encode()).hexdigest()



import json
import os
import uuid

from aiocache import cached
from fastapi_mongo_base.utils.aionetwork import aio_request
from fastapi_mongo_base.utils.basic import try_except_wrapper
from usso.session import AsyncUssoSession

from .schemas import AppAuth, BusinessSchema, Config

try:
    from server.config import Settings
except ImportError:
    from .core.config import Settings


class Business(BusinessSchema):
    @property
    def root_url(self):
        if self.domain.startswith("http"):
            return self.domain
        return f"https://{self.domain}"

    @classmethod
    @cached(ttl=60 * 10)
    @try_except_wrapper
    async def _get_query(
        cls,
        name: str = None,
        origin: str = None,
        user_id: uuid.UUID = None,
        uid: uuid.UUID = None,
        offset: int = 0,
        limit: int = 10,
        *args,
        **kwargs,
    ):
        params = {"offset": offset, "limit": limit}
        if user_id:
            params["user_id"] = str(user_id)
        if name:
            params["name"] = name
        if origin:
            params["origin"] = origin
        if uid:
            params["uid"] = str(uid)

        access_token = await cls.cls_access_token()
        headers = {"Authorization": f"Bearer {access_token}"}
        return await aio_request(
            method="get",
            url=Settings.business_domains_url,
            params=params,
            headers=headers,
        )

    @classmethod
    async def get_query(
        cls,
        name: str = None,
        origin: str = None,
        user_id: uuid.UUID = None,
        uid: uuid.UUID = None,
        offset: int = 0,
        limit: int = 10,
        *args,
        **kwargs,
    ):
        return (
            await cls._get_query(
                name=name,
                origin=origin,
                user_id=user_id,
                uid=uid,
                offset=offset,
                limit=limit,
                *args,
                **kwargs,
            )
            or {}
        )

    @classmethod
    async def get_with_query(cls, name: str = None, origin: str = None):
        businesses_dict = await cls.get_query(name=name, origin=origin)
        if not businesses_dict:
            return
        businesses_list = businesses_dict.get("items")
        if not businesses_list:
            return
        business = cls(**businesses_list[0])
        return business

    @classmethod
    async def get_by_origin(cls, origin: str):
        return await cls.get_with_query(origin=origin)

    @classmethod
    async def get_by_name(cls, name: str):
        return await cls.get_with_query(name=name)

    @classmethod
    async def list_items(
        cls,
        user_id: uuid.UUID = None,
        offset: int = 0,
        limit: int = 10,
        is_deleted: bool = False,
        *args,
        **kwargs,
    ) -> tuple[list["Business"], int]:
        business_dict = await cls.get_query(
            user_id=user_id,
            offset=offset,
            limit=limit,
            is_deleted=is_deleted,
            *args,
            **kwargs,
        )
        return [cls(**item) for item in business_dict.get("items", [])]

    @classmethod
    async def total_count(
        cls,
        user_id: uuid.UUID = None,
        is_deleted: bool = False,
        *args,
        **kwargs,
    ):
        business_dict = await cls.get_query(
            user_id=user_id,
            is_deleted=is_deleted,
            *args,
            **kwargs,
        )
        return business_dict.get("total", 0)

    @classmethod
    async def list_total_combined(
        cls,
        user_id: uuid.UUID = None,
        offset: int = 0,
        limit: int = 10,
        is_deleted: bool = False,
        *args,
        **kwargs,
    ) -> tuple[list["Business"], int]:
        return await cls.list_items(
            user_id=user_id,
            offset=offset,
            limit=limit,
            is_deleted=is_deleted,
            *args,
            **kwargs,
        ), await cls.total_count(
            user_id=user_id, is_deleted=is_deleted, *args, **kwargs
        )

    @classmethod
    async def get_item(cls, uid: uuid.UUID, user_id: uuid.UUID = None, *args, **kwargs):
        business_dict = await cls.get_query(uid=uid, user_id=user_id, *args, **kwargs)
        businesses_list = business_dict.get("items", [])
        if not businesses_list:
            return
        business = cls(**businesses_list[0])
        return business

    @classmethod
    @cached(ttl=getattr(Settings, "app_auth_expiry", 60))
    async def cls_access_token(cls):
        if os.getenv("USSO_ADMIN_API_KEY") and cls.cls_refresh_url():
            client = AsyncUssoSession(
                usso_refresh_url=cls.cls_refresh_url(),
                usso_api_key=os.getenv("USSO_ADMIN_API_KEY"),
                user_id=getattr(Settings, "USSO_USER_ID", None),
                api_key=None,
            )
            return client.access_token

        if os.getenv("USSO_REFRESH_TOKEN") and cls.cls_refresh_url():
            client = AsyncUssoSession(
                usso_refresh_url=cls.cls_refresh_url(),
                refresh_token=os.getenv("USSO_REFRESH_TOKEN"),
                api_key=None,
            )
            return client.access_token

        if hasattr(Settings, "app_id") and hasattr(Settings, "app_secret"):
            scopes = json.loads(getattr(Settings, "app_scopes", "[]"))
            app_auth = AppAuth(
                app_id=Settings.app_id,
                scopes=scopes,
                sso_url=Config().core_sso_url,
            )
            app_auth.secret = app_auth.get_secret(app_secret=Settings.app_secret)

            response_data: dict = await aio_request(
                method="post", url=Config().core_sso_url, json=app_auth.model_dump()
            )
            return response_data.get("access_token")

        raise ValueError(
            "USSO_ADMIN_API_KEY or USSO_REFRESH_TOKEN or app_id/app_secret are not set in settings."
        )

    @cached(ttl=getattr(Settings, "app_auth_expiry", 60))
    async def get_access_token(self):
        if hasattr(Settings, "app_id") and hasattr(Settings, "app_secret"):
            scopes = json.loads(getattr(Settings, "app_scopes", "[]"))
            app_auth = AppAuth(
                app_id=Settings.app_id,
                scopes=scopes,
                sso_url=self.config.sso_url,
            )
            app_auth.secret = app_auth.get_secret(app_secret=Settings.app_secret)

            response_data: dict = await aio_request(
                method="post", url=self.config.core_sso_url, json=app_auth.model_dump()
            )
            return response_data.get("access_token")

        return await self.cls_access_token()



import uuid
from typing import Literal

from fastapi import Request
from pydantic import BaseModel
from usso import UserData
from usso.fastapi import jwt_access_security, jwt_access_security_None

from .models import Business

try:
    from core.exceptions import BaseHTTPException
except ImportError:
    from fastapi_mongo_base.core.exceptions import BaseHTTPException


class AuthorizationData(BaseModel):
    user: UserData | None = None
    user_id: uuid.UUID | None = None
    business: Business | None = None
    issuer_type: Literal["Business", "User", "App", "Anonymous"] | None = None
    authorized: bool = False
    app_id: str | None = None

    scopes: list[str] | None = None


class AuthorizationException(BaseHTTPException):
    def __init__(self, message: str):
        super().__init__(401, "authorization_error", message)


async def get_business(
    request: Request,
) -> Business:
    business = await Business.get_by_origin(request.url.hostname)
    if not business:
        raise BaseHTTPException(404, "business_not_found", "business not found")
    return business


async def authorized_request(request: Request, scope: str = None) -> bool:
    # TODO Implement authorization logic
    # check scopes
    return True


async def get_request_body_dict(request: Request):
    body_bytes = await request.body()
    if not body_bytes:
        return {}
    return await request.json()


async def authorization_middleware(
    request: Request, anonymous_accepted=False
) -> AuthorizationData:
    authorization = AuthorizationData()

    authorization.business = await get_business(request)
    if anonymous_accepted:
        authorization.user = jwt_access_security_None(
            request, jwt_config=authorization.business.config.jwt_config
        )
    else:
        authorization.user = jwt_access_security(
            request, jwt_config=authorization.business.config.jwt_config
        )

    if authorization.user and authorization.user.authentication_method == "app":
        authorization.issuer_type = "App"
        authorization.user_id = (
            (
                request.query_params.get("user_id")
                or request.path_params.get("user_id")
                or (await get_request_body_dict(request)).get("user_id")
                or authorization.user.data.get("app_id")
            )
            if authorization.issuer_type == "App"  # check scopes
            else authorization.user.data.get("app_id")
        )
        authorization.app_id = authorization.user.data.get("app_id")
        authorization.scopes = authorization.user.data.get("scopes")

    elif (
        authorization.user and authorization.business.user_id == authorization.user.uid
    ):
        authorization.issuer_type = "Business"
        authorization.user_id = (
            request.query_params.get("user_id")
            or request.path_params.get("user_id")
            or (await get_request_body_dict(request)).get("user_id")
            or authorization.business.user_id
        )
    elif authorization.user:
        authorization.issuer_type = "User"
        authorization.user_id = authorization.user.uid
    else:
        authorization.issuer_type = "Anonymous"

    # authorization.app_id = request.headers.get("X-App-Id")
    authorization.authorized = await authorized_request(request)

    return authorization
